package combinatorParsing2

trait Position {

  /** The line number referred to by the position; line numbers start at 1 */
  def line: int

  /** The column number referred to by the position; column numbers start at 1 */
  def column: int

  /** The contents of line numbered `lnum' */
  protected def lineContents(lnum: int): String
  
  /** Returns string of the form `line.column' */
  override def toString = ""+line+"."+column

  /** Returns long form of position as a string: 
   *  The contents of the source line referred to it, with an ^ symbol
   *  in the following line pointing out the column
   *  Example:
   *
   *    List(this, is, an, example, line)
   *             ^
   */
  def longString = lineContents(line)+"\n"+indent(column)+"^"

  private def indent(col: int) = {
    val chars = new StringBuffer
    for (val i <- 1 until col) chars append ' '
    chars.toString
  }

  def <(that: Position) = {
    this.line < that.line || 
    this.line == that.line && this.column < that.column
  }
}
