package combinatorParsing2
import CharArrayReader.EofCh
import collection.mutable.HashSet

class StdLexical extends Scanners with StdTokens {
  
  def letter = elem("letter", .isLetter)
  def digit = elem("digit", .isDigit)
  def chrExcept(cs: char*) = elem("", ch => (cs forall (ch !=)))
  def whitespaceChar = elem("space char", ch => ch <= ' ' && ch != EofCh)
  def specialChar = elem("special char", ch => specialChars contains ch)

  def token: Parser[Token] = 
    ( letter ++ rep( letter | digit )                     ^^ (&mkString2 andThen processIdent)
    | digit ++ rep( digit )                               ^^ (&mkString2 andThen NumericLit)
    | '\'' ++ rep( chrExcept('\'', '\n', EofCh) ) ++ '\'' ^^ (&mkString andThen StringLit)
    | '\"' ++ rep( chrExcept('\"', '\n', EofCh) ) ++ '\"' ^^ (&mkString andThen StringLit)
    | '('                                                 ^^ Keyword("(")
    | ')'                                                 ^^ Keyword(")")
    | '=' ++ '>'                                          ^^ Keyword("=>")
    | EofCh                                               ^^ EOF
    | '\'' ++ failure("unclosed string literal")
    | '\"' ++ failure("unclosed string literal")
    | specialChar                                         ^^ { ch => Keyword(ch.toString) }
    | failure("illegal character")
    )

  def whitespace: Parser[Any] = rep(
      whitespaceChar
    | '/' ++ '*' ++ comment
    | '/' ++ '/' ++ rep( chrExcept(EofCh, '\n') )
    | '/' ++ '*' ++ failure("unclosed comment")
    )

  def comment: Parser[Any] = (
      '*' ++ '/'  ^^ ' ' 
    | chrExcept(EofCh) ++ comment
    )

  val reserved = new HashSet[String]
  val specialChars = new HashSet[Char]

  def mkString(cs: List[char]): String = cs.mkString("", "", "")

  def mkString2(p: Pair[char, List[char]]): String = mkString(p._1 :: p._2)

  def processIdent(name: String) = 
    if (reserved contains name) Keyword(name) else Identifier(name)
}
